@@ -8,66 +8,68 @@
 # Copyright (c) 1998-2000 by Ajuba Solutions.
 # All rights reserved.
 # 
-# RCS: @(#) $Id: all.tcl,v 1.17 2004/02/14 05:59:20 andreas_kupries Exp $
+# RCS: @(#) $Id: all.tcl,v 1.30 2005/09/28 04:51:19 andreas_kupries Exp $
+
+catch {wm withdraw .}
 
 set old_auto_path $auto_path
 
 if {[lsearch [namespace children] ::tcltest] == -1} {
     namespace eval ::tcltest {}
     proc ::tcltest::processCmdLineArgsAddFlagsHook {} {
-	return [list -modules]
+    return [list -modules]
     }
     proc ::tcltest::processCmdLineArgsHook {argv} {
-	array set foo $argv
-	catch {set ::modules $foo(-modules)}
+    array set foo $argv
+    catch {set ::modules $foo(-modules)}
     }
     proc ::tcltest::cleanupTestsHook {{c {}}} {
-	if { [string equal $c ""] } {
-	    return
-	}
-	# Get total/pass/skip/fail counts
-	array set foo [$c eval {array get ::tcltest::numTests}]
-	foreach index [list "Total" "Passed" "Skipped" "Failed"] {
-	    incr ::tcltest::numTests($index) $foo($index)
-	}
-	incr ::tcltest::numTestFiles
+    if { [string equal $c ""] } {
+        return
+    }
+    # Get total/pass/skip/fail counts
+    array set foo [$c eval {array get ::tcltest::numTests}]
+    foreach index [list "Total" "Passed" "Skipped" "Failed"] {
+        incr ::tcltest::numTests($index) $foo($index)
+    }
+    incr ::tcltest::numTestFiles
 
-	# Append the list of failFiles if necessary
-	set f [$c eval {
-	    set ff $::tcltest::failFiles
-	    if {($::tcltest::currentFailure) && \
-		    ([lsearch -exact $ff $testFileName] == -1)} {
-		set res [file join $::tcllibModule $testFileName]
-	    } else {
-		set res ""
-	    }
-	    set res
-	}] ; # {}
-	if { ![string equal $f ""] } {
-	    lappend ::tcltest::failFiles $f
-	}
+    # Append the list of failFiles if necessary
+    set f [$c eval {
+        set ff $::tcltest::failFiles
+        if {($::tcltest::currentFailure) && \
+            ([lsearch -exact $ff $testFileName] == -1)} {
+        set res [file join $::tcllibModule $testFileName]
+        } else {
+        set res ""
+        }
+        set res
+    }] ; # {}
+    if { ![string equal $f ""] } {
+        lappend ::tcltest::failFiles $f
+    }
 
-	# Get the "skipped because" information
-	unset foo
-	array set foo [$c eval {array get ::tcltest::skippedBecause}]
-	foreach constraint [array names foo] {
-	    if { ![info exists ::tcltest::skippedBecause($constraint)] } {
-		set ::tcltest::skippedBecause($constraint) $foo($constraint)
-	    } else {
-		incr ::tcltest::skippedBecause($constraint) $foo($constraint)
-	    }
-	}
+    # Get the "skipped because" information
+    unset foo
+    array set foo [$c eval {array get ::tcltest::skippedBecause}]
+    foreach constraint [array names foo] {
+        if { ![info exists ::tcltest::skippedBecause($constraint)] } {
+        set ::tcltest::skippedBecause($constraint) $foo($constraint)
+        } else {
+        incr ::tcltest::skippedBecause($constraint) $foo($constraint)
+        }
+    }
 
-	# Clean out the state in the slave
-	$c eval {
-	    foreach index [list "Total" "Passed" "Skipped" "Failed"] {
-		set ::tcltest::numTests($index) 0
-	    }
-	    set ::tcltest::failFiles {}
-	    foreach constraint [array names ::tcltest::skippedBecause] {
-		unset ::tcltest::skippedBecause($constraint)
-	    }
-	}
+    # Clean out the state in the slave
+    $c eval {
+        foreach index [list "Total" "Passed" "Skipped" "Failed"] {
+        set ::tcltest::numTests($index) 0
+        }
+        set ::tcltest::failFiles {}
+        foreach constraint [array names ::tcltest::skippedBecause] {
+        unset ::tcltest::skippedBecause($constraint)
+        }
+    }
     }
 
     package require tcltest
@@ -122,7 +124,7 @@ set old_apath $auto_path
 
 if {![info exists modules]} then {
     foreach module [glob [file join $root modules]/*/*.test] {
-	set tmp([lindex [file split $module] end-1]) 1
+    set tmp([lindex [file split $module] end-1]) 1
     }
     set modules [array names tmp]
     unset tmp
@@ -152,7 +154,10 @@ foreach module $modules {
 	# The next command allows the execution of 'tk' constrained
 	# tests, if Tk is present (for example when this code is run
 	# run by 'wish').
-	catch {package require Tk}
+	catch {
+	    package require Tk
+	    wm withdraw .
+	}
 
 	package require tcltest
 	namespace import ::tcltest::*
@@ -171,7 +176,12 @@ foreach module $modules {
 		append msg "\"[lindex $argList $missingIndex]\" to "
 		append msg "\"$functionName\""
 	    } else {
-		set msg "wrong # args: should be \"$functionName $argList\""
+		if {[package vcompare [package provide Tcl] 8.5] >= 0 
+		    && [string match args [lindex $argList end]]} {
+		    set argList [lreplace $argList end end ...]
+		}
+		if {$argList != {}} {set argList " $argList"}
+		set msg "wrong # args: should be \"$functionName$argList\""
 	    }
 	    return $msg
 	}
@@ -181,7 +191,16 @@ foreach module $modules {
 	    if {[package vcompare [package provide Tcl] 8.4] < 0} {
 		set msg "called \"$functionName\" with too many arguments"
 	    } else {
-		set msg "wrong # args: should be \"$functionName $argList\""
+		# create a different message for functions with no args
+		if {[llength $argList]} {
+		    if {[package vcompare [package provide Tcl] 8.5] >= 0 
+			&& [string match args [lindex $argList end]]} {
+			set argList [lreplace $argList end end ...]
+		    }
+		    set msg "wrong # args: should be \"$functionName $argList\""
+		} else {
+		    set msg "wrong # args: should be \"$functionName\""
+		}
 	    }
 	    return $msg
 	}
@@ -190,13 +209,13 @@ foreach module $modules {
 	if {[package vsatisfies [package provide tcltest] 2.0]} {
 	    # tcltest2.0+ has an API to specify a test constraint
 	    ::tcltest::testConstraint tcl8.3only \
-		    [expr {![package vsatisfies [package provide Tcl] 8.4]}]
+		[expr {![package vsatisfies [package provide Tcl] 8.4]}]
 	    ::tcltest::testConstraint tcl8.3plus \
-		    [expr {[package vsatisfies [package provide Tcl] 8.3]}]
+		[expr {[package vsatisfies [package provide Tcl] 8.3]}]
 	    ::tcltest::testConstraint tcl8.4plus \
-		    [expr {[package vsatisfies [package provide Tcl] 8.4]}]
+		[expr {[package vsatisfies [package provide Tcl] 8.4]}]
 	    ::tcltest::testConstraint tcl8.5plus \
-		    [expr {[package vsatisfies [package provide Tcl] 8.5]}]
+		[expr {[package vsatisfies [package provide Tcl] 8.5]}]
 
 	    proc ::tcltest::queryConstraint {c} {
 		return [testConstraint $c]
@@ -204,14 +223,14 @@ foreach module $modules {
 	} else {
 	    # In tcltest1.0, a global variable needs to be set directly.
 	    set ::tcltest::testConstraints(tcl8.3only) \
-		    [expr {![package vsatisfies [package provide Tcl] 8.4]}]
+		[expr {![package vsatisfies [package provide Tcl] 8.4]}]
 	    set ::tcltest::testConstraints(tcl8.3plus) \
-		    [expr {[package vsatisfies [package provide Tcl] 8.3]}]
+		[expr {[package vsatisfies [package provide Tcl] 8.3]}]
 	    set ::tcltest::testConstraints(tcl8.4plus) \
-		    [expr {[package vsatisfies [package provide Tcl] 8.4]}]
+		[expr {[package vsatisfies [package provide Tcl] 8.4]}]
 	    set ::tcltest::testConstraints(tcl8.5plus) \
-		    [expr {[package vsatisfies [package provide Tcl] 8.5]}]
-
+		[expr {[package vsatisfies [package provide Tcl] 8.5]}]
+	    
 	    proc ::tcltest::queryConstraint {c} {
 		if {![info exists ::tcltest::testConstraints($c)]} {
 		    # Not existing constraints are not set.
@@ -222,7 +241,7 @@ foreach module $modules {
 	}
     }
     interp alias $c ::tcltest::cleanupTestsHook {} \
-	    ::tcltest::cleanupTestsHook $c
+        ::tcltest::cleanupTestsHook $c
     # source each of the specified tests
     foreach file [lsort [::tcltest::getMatchingFiles]] {
 	set tail [file tail $file]
