@@ -8,9 +8,9 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: base64.tcl,v 1.26 2005/09/30 05:36:38 andreas_kupries Exp $
 
-# Version 1.0   implemented Base64_Encode, Bae64_Decode
+# Version 1.0   implemented Base64_Encode, Base64_Decode
 # Version 2.0   uses the base64 namespace
 # Version 2.1   fixes various decode bugs and adds options to encode
 # Version 2.2   is much faster, Tcl8.0 compatible
@@ -42,7 +42,7 @@ if {![catch {package require Trf 2.0}]} {
     
     proc ::base64::encode {args} {
 	# Set the default wrapchar and maximum line length to match the output
-	# of GNU uuencode 4.2.  Various RFC's allow for different wrapping 
+	# of GNU uuencode 4.2.  Various RFCs allow for different wrapping 
 	# characters and wraplengths, so these may be overridden by command line
 	# options.
 	set wrapchar "\n"
@@ -182,7 +182,7 @@ if {![catch {package require Trf 2.0}]} {
 	set base64_en $::base64::base64_en
 	
 	# Set the default wrapchar and maximum line length to match the output
-	# of GNU uuencode 4.2.  Various RFC's allow for different wrapping 
+	# of GNU uuencode 4.2.  Various RFCs allow for different wrapping 
 	# characters and wraplengths, so these may be overridden by command line
 	# options.
 	set wrapchar "\n"
@@ -302,7 +302,6 @@ if {![catch {package require Trf 2.0}]} {
 		
 		foreach {v w z} $nums break
 		set a [expr {($v << 2) | (($w & 0x30) >> 4)}]
-		
 		if {$z == {}} {
 		    append output [binary format c $a ]
 		} else {
@@ -322,4 +321,4 @@ if {![catch {package require Trf 2.0}]} {
     }
 }
 
-package provide base64 2.3
+package provide base64 2.3.2
