@@ -12,25 +12,34 @@
 # Support for SPF (http://spf.pobox.com/rfcs.html) will need updating
 # if or when the proposed draft becomes accepted.
 #
+# Support added for RFC1886 - DNS Extensions to support IP version 6
+# Support added for RFC2782 - DNS RR for specifying the location of services
+# Support added for RFC1995 - Incremental Zone Transfer in DNS
+#
 # TODO:
 #  - When using tcp we should make better use of the open connection and
 #    send multiple queries along the same connection.
 #
+#  - We must switch to using TCP for truncated UDP packets.
+#
+#  - Read RFC 2136 - dynamic updating of DNS
+#
 # -------------------------------------------------------------------------
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id$
+# $Id: dns.tcl,v 1.30 2005/09/30 05:36:38 andreas_kupries Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 package require logger;                 # tcllib 1.3
 package require uri;                    # tcllib 1.1
 package require uri::urn;               # tcllib 1.2
+package require ip;                     # tcllib 1.7
 
 namespace eval ::dns {
-    variable version 1.1
-    variable rcsid {$Id$}
+    variable version 1.3.0
+    variable rcsid {$Id: dns.tcl,v 1.30 2005/09/30 05:36:38 andreas_kupries Exp $}
 
     namespace export configure resolve name address cname \
         status reset wait cleanup errorcode
@@ -46,19 +55,25 @@ namespace eval ::dns {
             loglevel   warn
         }
         variable log [logger::init dns]
-        ${log}::enable $options(loglevel)
+        ${log}::setlevel $options(loglevel)
     }
 
+    # We can use either ceptcl or tcludp for UDP support.
     if {![catch {package require udp 1.0.4} msg]} { ;# tcludp 1.0.4+
         # If TclUDP 1.0.4 or better is available, use it.
         set options(protocol) udp
+    } else {
+        if {![catch {package require ceptcl} msg]} {
+            set options(protocol) udp
+        }
     }
 
     variable types
     array set types { 
         A 1  NS 2  MD 3  MF 4  CNAME 5  SOA 6  MB 7  MG 8  MR 9 
         NULL 10  WKS 11  PTR 12  HINFO 13  MINFO 14  MX 15  TXT 16
-        SPF 16 AXFR 252  MAILB 253  MAILA 254  * 255
+        SPF 16 AAAA 28 SRV 33 IXFR 251 AXFR 252  MAILB 253  MAILA 254
+        * 255
     } 
 
     variable classes
@@ -142,7 +157,7 @@ proc ::dns::configure {args} {
                     return $options(loglevel)
                 } else {
                     set options(loglevel) [Pop args 1]
-                    ${log}::enable $options(loglevel)
+                    ${log}::setlevel $options(loglevel)
                 }
             }
             --    { Pop args ; break }
@@ -174,12 +189,14 @@ proc ::dns::resolve {query args} {
     while {[info exists [set token [namespace current]::$id]]} {
         set id [incr uid]
     }
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
     # Setup token/state defaults.
     set state(id)          $id
     set state(query)       $query
+    set state(qdata)       ""
     set state(opcode)      0;                   # 0 = query, 1 = inverse query.
     set state(-type)       A;                   # DNS record type (A address)
     set state(-class)      IN;                  # IN (internet address space)
@@ -217,6 +234,7 @@ proc ::dns::resolve {query args} {
             -re*  { set state(-recurse) [Pop args 1] }
             -inv* { set state(opcode) 1 }
             -status {set state(opcode) 2}
+            -data { set state(qdata) [Pop args 1] }
             default {
                 set opts [join [lsort [array names state -*]] ", "]
                 return -code error "bad option [lindex $args 0]: \
@@ -231,13 +249,15 @@ proc ::dns::resolve {query args} {
     }
 
     if {$state(-protocol) == "udp"} {
-        if {[package provide udp] == {}} {
-            return -code error "udp support is not available, get tcludp"
+        if {[llength [package provide ceptcl]] == 0 \
+                && [llength [package provide udp]] == 0} {
+            return -code error "udp support is not available,\
+                get ceptcl or tcludp"
         }
     }
-
+    
     # Check for reverse lookups
-    if {[regexp {(?:\d{0,3}\.){3}\d{0,3}} $state(query)]} {
+    if {[regexp {^(?:\d{0,3}\.){3}\d{0,3}$} $state(query)]} {
         set addr [lreverse [split $state(query) .]]
         lappend addr in-addr arpa
         set state(query) [join $addr .]
@@ -311,8 +331,13 @@ proc ::dns::address {token} {
         array set AN $answer
 
         if {[info exists AN(type)]} {
-            if {$AN(type) == "A"} {
-                lappend r $AN(rdata)
+            switch -exact -- $AN(type) {
+                "A" {
+                    lappend r $AN(rdata)
+                }
+                "AAAA" {
+                    lappend r $AN(rdata)
+                }
             }
         }
     }
@@ -340,8 +365,8 @@ proc ::dns::cname {token} {
 # Description:
 #   Return the decoded answer records. This can be used for more complex
 #   queries where the answer isn't supported byb cname/address/name.
-proc ::dns::result {token} {
-    array set reply [Decode $token]
+proc ::dns::result {token args} {
+    array set reply [eval [linsert $args 0 Decode $token]]
     return $reply(AN)
 }
 
@@ -351,6 +376,7 @@ proc ::dns::result {token} {
 #  Get the status of the request.
 #
 proc ::dns::status {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     return $state(status)
@@ -360,6 +386,7 @@ proc ::dns::status {token} {
 #  Get the error message. Empty if no error.
 #
 proc ::dns::error {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     if {[info exists state(error)]} {
@@ -372,6 +399,7 @@ proc ::dns::error {token} {
 #  Get the error code. This is 0 for a successful transaction.
 #
 proc ::dns::errorcode {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     set flags [Flags $token]
@@ -384,6 +412,7 @@ proc ::dns::errorcode {token} {
 #  Reset a connection with optional reason.
 #
 proc ::dns::reset {token {why reset} {errormsg {}}} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     set state(status) $why
@@ -398,6 +427,7 @@ proc ::dns::reset {token {why reset} {errormsg {}}} {
 #  Wait for a request to complete and return the status.
 #
 proc ::dns::wait {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
@@ -412,6 +442,7 @@ proc ::dns::wait {token} {
 #  Remove any state associated with this token.
 #
 proc ::dns::cleanup {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     if {[info exists state]} {
@@ -432,9 +463,11 @@ proc ::dns::dump {args} {
         set type [lindex $args 0]
         set token [lindex $args 1]
     } else {
-        error "wrong # args: should be \"dump ?option? methodName\""
+        return -code error "wrong # args:\
+            should be \"dump ?option? methodName\""
     }
 
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     
@@ -474,6 +507,7 @@ proc ::dns::DumpMessage {data} {
 #  Contruct a DNS query packet.
 #
 proc ::dns::BuildMessage {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     variable types
@@ -490,31 +524,33 @@ proc ::dns::BuildMessage {token} {
 
     set qdcount 0
     set qsection {}
+    set nscount 0
+    set nsdata {}
 
     # In theory we can send multiple queries. In practice, named doesn't
     # appear to like that much. If it did work we'd do this:
     #  foreach domain [linsert $options(search) 0 {}] ...
 
-    set qname [string trim $state(query) .]
-    
-    # break up the name into length tagged 'labels'
-    foreach part [split $qname .] {
-        set label [binary format c [string length $part]]
-        append qsection $label $part
-    }
-    # append the root label and the type flag and query class.
-    append qsection [binary format cSS 0 \
-            $types($state(-type))\
-            $classes($state(-class))]
+
+    # Pack the query: QNAME QTYPE QCLASS
+    set qsection [PackName $state(query)]
+    append qsection [binary format SS \
+                         $types($state(-type))\
+                         $classes($state(-class))]
     incr qdcount
 
+    if {[string length $state(qdata)] > 0} {
+        set nsdata [eval [linsert $state(qdata) 0 PackRecord]]
+        incr nscount
+    }
+
     switch -exact -- $state(opcode) {
         0 {
             # QUERY
             set state(request) [binary format SSSSSS $state(id) \
                 [expr {($state(opcode) << 11) | ($state(-recurse) << 8)}] \
-                $qdcount 0 0 0]
-            append state(request) $qsection
+                                    $qdcount 0 $nscount 0]
+            append state(request) $qsection $nsdata
         }
         1 {
             # IQUERY            
@@ -546,12 +582,91 @@ proc ::dns::BuildMessage {token} {
     return
 }
 
+# Pack a human readable dns name into a DNS resource record format.
+proc ::dns::PackName {name} {
+    set data ""
+    foreach part [split [string trim $name .] .] {
+        set len [string length $part]
+        append data [binary format ca$len $len $part]
+    }
+    append data \x00
+    return $data
+}
+
+# Pack a character string - byte length prefixed
+proc ::dns::PackString {text} {
+    set len [string length $text]
+    set data [binary format ca$len $len $text]
+    return $data
+}
+
+# Pack up a single DNS resource record. See RFC1035: 3.2 for the format
+# of each type.
+# eg: PackRecord name wiki.tcl.tk type MX class IN rdata {10 mail.example.com}
+#
+proc ::dns::PackRecord {args} {
+    variable types
+    variable classes
+    array set rr {name "" type A class IN ttl 0 rdlength 0 rdata ""}
+    array set rr $args
+    set data [PackName $rr(name)]
+
+    switch -exact -- $rr(type) {
+        CNAME - MB - MD - MF - MG - MR - NS - PTR {
+            set rr(rdata) [PackName $rr(rdata)] 
+        }
+        HINFO { 
+            array set r {CPU {} OS {}}
+            array set r $rr(rdata)
+            set rr(rdata) [PackString $r(CPU)]
+            append rr(rdata) [PackString $r(OS)]
+        }
+        MINFO {
+            array set r {RMAILBX {} EMAILBX {}}
+            array set r $rr(rdata)
+            set rr(rdata) [PackString $r(RMAILBX)]
+            append rr(rdata) [PackString $r(EMAILBX)]
+        }
+        MX {
+            foreach {pref exch} $rr(rdata) break
+            set rr(rdata) [binary format S $pref]
+            append rr(rdata) [PackName $exch]
+        }
+        TXT {
+            set str $rr(rdata)
+            set len [string length [set str $rr(rdata)]]
+            set rr(rdata) ""
+            for {set n 0} {$n < $len} {incr n} {
+                set s [string range $str $n [incr n 253]]
+                append rr(rdata) [PackString $s]
+            }
+        }          
+        NULL {}
+        SOA {
+            array set r {MNAME {} RNAME {}
+                SERIAL 0 REFRESH 0 RETRY 0 EXPIRE 0 MINIMUM 0}
+            array set r $rr(rdata)
+            set rr(rdata) [PackName $r(MNAME)]
+            append rr(rdata) [PackName $r(RNAME)]
+            append rr(rdata) [binary format IIIII $r(SERIAL) \
+                                  $r(REFRESH) $r(RETRY) $r(EXPIRE) $r(MINIMUM)]
+        }
+    }
+
+    # append the root label and the type flag and query class.
+    append data [binary format SSIS $types($rr(type)) \
+                     $classes($rr(class)) $rr(ttl) [string length $rr(rdata)]]
+    append data $rr(rdata)
+    return $data
+}
+
 # -------------------------------------------------------------------------
 
 # Description:
 #  Transmit a DNS request over a tcp connection.
 #
 proc ::dns::TcpTransmit {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
@@ -589,6 +704,7 @@ proc ::dns::TcpTransmit {token} {
 #  cannot do this.
 #
 proc ::dns::UdpTransmit {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
@@ -600,8 +716,15 @@ proc ::dns::UdpTransmit {token} {
                                   "operation timed out"]]
     }
     
-    set state(sock) [udp_open]
-    udp_conf $state(sock) $state(-nameserver) $state(-port)
+    if {[llength [package provide ceptcl]] > 0} {
+        # using ceptcl
+        set state(sock) [cep -type datagram $state(-nameserver) $state(-port)]
+        fconfigure $state(sock) -blocking 0
+    } else {
+        # using tcludp
+        set state(sock) [udp_open]
+        udp_conf $state(sock) $state(-nameserver) $state(-port)
+    }
     fconfigure $state(sock) -translation binary -buffering none
     set state(status) connect
     puts -nonewline $state(sock) $state(request)
@@ -617,6 +740,7 @@ proc ::dns::UdpTransmit {token} {
 #  Tidy up after a tcp transaction.
 #
 proc ::dns::Finish {token {errormsg ""}} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     global errorInfo errorCode
@@ -646,6 +770,7 @@ proc ::dns::Finish {token {errormsg ""}} {
 #  Handle end-of-file on a tcp connection.
 #
 proc ::dns::Eof {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     set state(status) eof
@@ -658,6 +783,7 @@ proc ::dns::Eof {token} {
 #  Process a DNS reply packet (protocol independent)
 #
 proc ::dns::Receive {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
@@ -687,6 +813,7 @@ proc ::dns::Receive {token} {
 #
 proc ::dns::TcpEvent {token} {
     variable log
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     set s $state(sock)
@@ -699,7 +826,7 @@ proc ::dns::TcpEvent {token} {
     set status [catch {read $state(sock)} result]
     if {$status != 0} {
         ${log}::debug "Event error: $result"
-        Finish $tok "error reading data: $result"
+        Finish $token "error reading data: $result"
     } elseif { [string length $result] >= 0 } {
         if {[catch {
             # Handle incomplete reads - check the size and keep reading.
@@ -719,7 +846,7 @@ proc ::dns::TcpEvent {token} {
                    [string length $state(reply)] should be $state(size)"
             }
         } err]} {
-            Finish $tok "Event error: $err"
+            Finish $token "Event error: $err"
         }
     } elseif { [eof $state(sock)] } {
         Eof $token
@@ -727,7 +854,7 @@ proc ::dns::TcpEvent {token} {
         ${log}::debug "Event blocked"
     } else {
         ${log}::critical "Event error: this can't happen!"
-        Finish $tok "Event error: this can't happen!"
+        Finish $token "Event error: this can't happen!"
     }
 }
 
@@ -736,6 +863,7 @@ proc ::dns::TcpEvent {token} {
 # Description:
 #  file event handler for udp sockets.
 proc ::dns::UdpEvent {token} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     set s $state(sock)
@@ -751,6 +879,7 @@ proc ::dns::UdpEvent {token} {
 # -------------------------------------------------------------------------
 
 proc ::dns::Flags {token {varname {}}} {
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
     
@@ -781,10 +910,28 @@ proc ::dns::Flags {token {varname {}}} {
 #
 proc ::dns::Decode {token args} {
     variable log
+    # FRINK: nocheck
     variable $token
     upvar 0 $token state
 
-    binary scan $state(reply) SSSSSSc* mid hdr nQD nAN nNS nAR data
+    array set opts {-rdata 0 -query 0}
+    while {[string match -* [set option [lindex $args 0]]]} {
+        switch -exact -- $option {
+            -rdata { set opts(-rdata) 1 }
+            -query { set opts(-query) 1 }
+            default {
+                return -code error "bad option \"$option\":\
+                    must be -rdata"
+            }
+        }
+        Pop args
+    }
+
+    if {$opts(-query)} {
+        binary scan $state(request) SSSSSSc* mid hdr nQD nAN nNS nAR data
+    } else {
+        binary scan $state(reply) SSSSSSc* mid hdr nQD nAN nNS nAR data
+    }
 
     set fResponse      [expr {($hdr & 0x8000) >> 15}]
     set fOpcode        [expr {($hdr & 0x7800) >> 11}]
@@ -815,11 +962,11 @@ proc ::dns::Decode {token args} {
     set r {}
     set QD [ReadQuestion $nQD $state(reply) ndx]
     lappend r QD $QD
-    set AN [ReadAnswer $nAN $state(reply) ndx]
+    set AN [ReadAnswer $nAN $state(reply) ndx $opts(-rdata)]
     lappend r AN $AN
-    set NS [ReadAnswer $nNS $state(reply) ndx]
+    set NS [ReadAnswer $nNS $state(reply) ndx $opts(-rdata)]
     lappend r NS $NS
-    set AR [ReadAnswer $nAR $state(reply) ndx]
+    set AR [ReadAnswer $nAR $state(reply) ndx $opts(-rdata)]
     lappend r AR $AR
     return $r
 }
@@ -907,7 +1054,7 @@ proc ::dns::ReadQuestion {nitems data indexvar} {
 
 # Read an answer section from a DNS message. 
 #
-proc ::dns::ReadAnswer {nitems data indexvar} {
+proc ::dns::ReadAnswer {nitems data indexvar {raw 0}} {
     variable types
     variable classes
     upvar $indexvar index
@@ -932,34 +1079,50 @@ proc ::dns::ReadAnswer {nitems data indexvar} {
         incr index 10
         set rdata [string range $data $index [expr {$index + $rdlength - 1}]]
 
-        switch -- $type {
-            A {
-                set rdata [join [Expand $rdata] .] 
-            }
-            NS - CNAME - PTR {
-                set rdata [ReadName data $index off] 
-            }
-            MX {
-                binary scan $rdata S preference
-                set exchange [ReadName data [expr {$index + 2}] off]
-                set rdata [list $preference $exchange]
-            }
-            SOA {
-                set x $index
-                set rdata [list MNAME [ReadName data $x off]]
-                incr x $off 
-                lappend rdata RNAME [ReadName data $x off]
-                incr x $off
-                lappend rdata SERIAL [ReadULong data $x off]
-                incr x $off
-                lappend rdata REFRESH [ReadLong data $x off]
-                incr x $off
-                lappend rdata RETRY [ReadLong data $x off]
-                incr x $off
-                lappend rdata EXPIRE [ReadLong data $x off]
-                incr x $off
-                lappend rdata MINIMUM [ReadULong data $x off]
-                incr x $off
+        if {! $raw} {
+            switch -- $type {
+                A {
+                    set rdata [join [Expand $rdata] .]
+                }
+                AAAA {
+                    set rdata [ip::contract [ip::ToString $rdata]]
+                }
+                NS - CNAME - PTR {
+                    set rdata [ReadName data $index off] 
+                }
+                MX {
+                    binary scan $rdata S preference
+                    set exchange [ReadName data [expr {$index + 2}] off]
+                    set rdata [list $preference $exchange]
+                }
+                SRV {
+                    set x $index
+                    set rdata [list priority [ReadUShort data $x off]]
+                    incr x $off
+                    lappend rdata weight [ReadUShort data $x off]
+                    incr x $off
+                    lappend rdata port [ReadUShort data $x off]
+                    incr x $off
+                    lappend rdata target [ReadName data $x off]
+                    incr x $off
+                }
+                SOA {
+                    set x $index
+                    set rdata [list MNAME [ReadName data $x off]]
+                    incr x $off 
+                    lappend rdata RNAME [ReadName data $x off]
+                    incr x $off
+                    lappend rdata SERIAL [ReadULong data $x off]
+                    incr x $off
+                    lappend rdata REFRESH [ReadLong data $x off]
+                    incr x $off
+                    lappend rdata RETRY [ReadLong data $x off]
+                    incr x $off
+                    lappend rdata EXPIRE [ReadLong data $x off]
+                    incr x $off
+                    lappend rdata MINIMUM [ReadULong data $x off]
+                    incr x $off
+                }
             }
         }
 
@@ -980,7 +1143,7 @@ proc ::dns::ReadLong {datavar index usedvar} {
     upvar $usedvar used
     set r {}
     set used 0
-    if {[binary scan [string range $data $index end] I r]} {
+    if {[binary scan $data @${index}I r]} {
         set used 4
     }
     return $r
@@ -991,10 +1154,24 @@ proc ::dns::ReadULong {datavar index usedvar} {
     upvar $usedvar used
     set r {}
     set used 0
-    if {[binary scan [string range $data $index end] cccc b1 b2 b3 b4]} {
+    if {[binary scan $data @${index}cccc b1 b2 b3 b4]} {
         set used 4
         # This gets us an unsigned value.
-        set r [expr {$b4 + ($b3 << 8) + ($b2 << 16) + ($b1 << 24)}] 
+        set r [expr {($b4 & 0xFF) + (($b3 & 0xFF) << 8) 
+                     + (($b2 & 0xFF) << 16) + ($b1 << 24)}] 
+    }
+    return $r
+}
+
+proc ::dns::ReadUShort {datavar index usedvar} {
+    upvar $datavar data
+    upvar $usedvar used
+    set r {}
+    set used 0
+    if {[binary scan [string range $data $index end] cc b1 b2]} {
+        set used 2
+        # This gets us an unsigned value.
+        set r [expr {(($b2 & 0xff) + (($b1 & 0xff) << 8)) & 0xffff}] 
     }
     return $r
 }
@@ -1036,41 +1213,61 @@ proc ::dns::ReadName {datavar index usedvar} {
 
 # -------------------------------------------------------------------------
 
-# Experimental support for finding the nameservers to use on a Windows
-# machine
+# Experimental support for finding the nameservers
+#
 # For unix we can just parse the /etc/resolv.conf if it exists.
 # Of couse, some unices use /etc/resolver and other things (NIS for instance)
+# On Windows, we can examine the Internet Explorer settings from the registry.
 #
-if {$::tcl_platform(platform) == "Windows"} {
-
-proc ::dns::Win32_NameServers {} {
-    package require registry
-    set base {HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\Tcpip}
-    set param "$base\\Parameters"
-    set interfaces "$param\\Interfaces"
-    set nameservers {}
-    AppendRegistryValue $param NameServer nameservers
-    AppendRegistryValue $param DhcpNameServer nameservers
-    foreach i [registry keys $interfaces] {
-        AppendRegistryValue "$interfaces\\$i" NameServer nameservers
-        AppendRegistryValue "$interfaces\\$i" DhcpNameServer nameservers
+switch -exact $::tcl_platform(platform) {
+    windows {
+        proc ::dns::nameservers {} {
+            package require registry
+            set base {HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\Tcpip}
+            set param "$base\\Parameters"
+            set interfaces "$param\\Interfaces"
+            set nameservers {}
+            AppendRegistryValue $param NameServer nameservers
+            AppendRegistryValue $param DhcpNameServer nameservers
+            foreach i [registry keys $interfaces] {
+                AppendRegistryValue "$interfaces\\$i" NameServer nameservers
+                AppendRegistryValue "$interfaces\\$i" DhcpNameServer nameservers
+            }
+            
+            return $nameservers
+        }
+        proc ::dns::AppendRegistryValue {key val listName} {
+            upvar $listName lst
+            if {![catch {registry get $key $val} v]} {
+                if {[lsearch -exact $lst $v] == -1} {
+                    set lst [concat $lst $v]
+                }
+            }
+        }
     }
-
-    # FIX ME: this doesn't preserve the original preference ordering
-    return [lsort -unique $nameservers]
-}
-
-
-proc ::dns::AppendRegistryValue {key val listName} {
-    upvar $listName lst
-    if {![catch {registry get $key $val} v]} {
-        set lst [concat $lst $v]
+    unix {
+        proc ::dns::nameservers {} {
+            set nameservers {}
+            if {[file readable /etc/resolv.conf]} {
+                set f [open /etc/resolv.conf r]
+                while {![eof $f]} {
+                    gets $f line
+                    if {[regexp {^\s*nameserver\s+(.*)$} $line -> ns]} {
+                        lappend nameservers $ns
+                    }
+                }
+                close $f
+            }
+            return $nameservers
+        }
+    }
+    default {
+        proc ::dns::nameservers {} {
+            return -code error "command not supported for this platform."
+        }
     }
 }
 
-}
-
-
 # -------------------------------------------------------------------------
 # Possible support for the DNS URL scheme.
 # Ref: http://www.ietf.org/internet-drafts/draft-josefsson-dns-url-04.txt
@@ -1111,7 +1308,7 @@ proc ::uri::SplitDns {uri} {
     array set parts {nameserver {} query {} class {} type {} port {}}
 
     # validate the uri
-    if {[regexp $dns::schemepart $uri r] == 1} {
+    if {[regexp -- $dns::schemepart $uri r] == 1} {
 
         # deal with the optional class and type specifiers
         if {[regexp -indices -- "${classOrTypeSpec}$" $uri range]} {
