@@ -2,12 +2,12 @@
 #
 #	Implementation of doctoc objects for Tcl.
 #
-# Copyright (c) 2003 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2005 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: doctoc.tcl,v 1.11 2005/09/30 05:36:39 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require textutil::expander
@@ -533,7 +533,7 @@ proc ::doctools::toc::LookupFormat {name format} {
 #	formatting engine
 #
 # Arguments:
-#	name	Name of the doctoc object to manipulaye
+#	name	Name of the doctoc object to manipulate
 #	format	Name of file containing the code of the engine
 #
 # Results:
@@ -636,7 +636,7 @@ proc ::doctools::toc::SetupFormatter {name format} {
 #	doctoc formatting commands
 #
 # Arguments:
-#	name	Name of the doctoc object to manipulaye
+#	name	Name of the doctoc object to manipulate
 #
 # Results:
 #	None.
@@ -692,7 +692,7 @@ proc ::doctools::toc::SetupChecker {name} {
 #	Create and initializes the expander for input
 #
 # Arguments:
-#	name	Name of the doctoc object to manipulaye
+#	name	Name of the doctoc object to manipulate
 #
 # Results:
 #	None.
@@ -731,7 +731,7 @@ proc ::doctools::toc::SearchPaths {name} {
 
 # ::doctools::toc::FmtError --
 #
-#	API for checker. Called when an error occured.
+#	API for checker. Called when an error occurred.
 #
 # Arguments:
 #	name	Name of the doctoc object to query.
@@ -902,4 +902,4 @@ namespace eval ::doctools::toc {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools::toc 0.2
+package provide doctools::toc 0.2.1
