@@ -16,10 +16,9 @@
 # Copyright (c) 2001 by ActiveState Tool Corp.
 # See the file license.terms.
 
-package require Tcl 8.2
-package require struct 1
-package require cmdline 1.1
-package provide htmlparse 0.2
+package require Tcl       8.2
+package require struct::stack
+package require cmdline   1.1
 
 namespace eval ::htmlparse {
     namespace export		\
@@ -31,24 +30,16 @@ namespace eval ::htmlparse {
 	    removeFormDefs
 
     # Table of escape characters. Maps from their names to the actual
-    # character.
+    # character.  See http://htmlhelp.org/reference/html40/entities/
 
-    variable escapes
+    variable namedEntities
 
-    # I. Standard escapes. (ISO latin-1 esc's are in a different table)
-
-    array set escapes {
-	lt <   gt >   amp &   quot \"   copy \xa9
-	reg \xae   ob \x7b   cb \x7d   nbsp \xa0
-    } ; # " make the emacs highlighting code happy.
-
-    # II. ISO Latin-1 escape codes
-
-    array set escapes {
+    # I. Latin-1 Entities (HTML 4.01)
+    array set namedEntities {
 	nbsp \xa0 iexcl \xa1 cent \xa2 pound \xa3 curren \xa4
 	yen \xa5 brvbar \xa6 sect \xa7 uml \xa8 copy \xa9
 	ordf \xaa laquo \xab not \xac shy \xad reg \xae
-	hibar \xaf deg \xb0 plusmn \xb1 sup2 \xb2 sup3 \xb3
+	macr \xaf deg \xb0 plusmn \xb1 sup2 \xb2 sup3 \xb3
 	acute \xb4 micro \xb5 para \xb6 middot \xb7 cedil \xb8
 	sup1 \xb9 ordm \xba raquo \xbb frac14 \xbc frac12 \xbd
 	frac34 \xbe iquest \xbf Agrave \xc0 Aacute \xc1 Acirc \xc2
@@ -67,6 +58,49 @@ namespace eval ::htmlparse {
 	yuml \xff
     }
 
+    # II. Entities for Symbols and Greek Letters (HTML 4.01)
+    array set namedEntities {
+	fnof \u192 Alpha \u391 Beta \u392 Gamma \u393 Delta \u394
+	Epsilon \u395 Zeta \u396 Eta \u397 Theta \u398 Iota \u399
+	Kappa \u39A Lambda \u39B Mu \u39C Nu \u39D Xi \u39E
+	Omicron \u39F Pi \u3A0 Rho \u3A1 Sigma \u3A3 Tau \u3A4
+	Upsilon \u3A5 Phi \u3A6 Chi \u3A7 Psi \u3A8 Omega \u3A9
+	alpha \u3B1 beta \u3B2 gamma \u3B3 delta \u3B4 epsilon \u3B5
+	zeta \u3B6 eta \u3B7 theta \u3B8 iota \u3B9 kappa \u3BA
+	lambda \u3BB mu \u3BC nu \u3BD xi \u3BE omicron \u3BF
+	pi \u3C0 rho \u3C1 sigmaf \u3C2 sigma \u3C3 tau \u3C4
+	upsilon \u3C5 phi \u3C6 chi \u3C7 psi \u3C8 omega \u3C9
+	thetasym \u3D1 upsih \u3D2 piv \u3D6 bull \u2022
+	hellip \u2026 prime \u2032 Prime \u2033 oline \u203E
+	frasl \u2044 weierp \u2118 image \u2111 real \u211C
+	trade \u2122 alefsym \u2135 larr \u2190 uarr \u2191
+	rarr \u2192 darr \u2193 harr \u2194 crarr \u21B5
+	lArr \u21D0 uArr \u21D1 rArr \u21D2 dArr \u21D3 hArr \u21D4
+	forall \u2200 part \u2202 exist \u2203 empty \u2205
+	nabla \u2207 isin \u2208 notin \u2209 ni \u220B prod \u220F
+	sum \u2211 minus \u2212 lowast \u2217 radic \u221A
+	prop \u221D infin \u221E ang \u2220 and \u2227 or \u2228
+	cap \u2229 cup \u222A int \u222B there4 \u2234 sim \u223C
+	cong \u2245 asymp \u2248 ne \u2260 equiv \u2261 le \u2264
+	ge \u2265 sub \u2282 sup \u2283 nsub \u2284 sube \u2286
+	supe \u2287 oplus \u2295 otimes \u2297 perp \u22A5
+	sdot \u22C5 lceil \u2308 rceil \u2309 lfloor \u230A
+	rfloor \u230B lang \u2329 rang \u232A loz \u25CA
+	spades \u2660 clubs \u2663 hearts \u2665 diams \u2666
+    }
+
+    # III. Special Entities (HTML 4.01)
+    array set namedEntities {
+	quot \x22 amp \x26 lt \x3C gt \x3E OElig \u152 oelig \u153
+	Scaron \u160 scaron \u161 Yuml \u178 circ \u2C6
+	tilde \u2DC ensp \u2002 emsp \u2003 thinsp \u2009
+	zwnj \u200C zwj \u200D lrm \u200E rlm \u200F ndash \u2013
+	mdash \u2014 lsquo \u2018 rsquo \u2019 sbquo \u201A
+	ldquo \u201C rdquo \u201D bdquo \u201E dagger \u2020
+	Dagger \u2021 permil \u2030 lsaquo \u2039 rsaquo \u203A
+	euro \u20AC
+    }
+
     # Internal cache for the foreach variable-lists and the
     # substitution strings used to split a HTML string into
     # incrementally handleable scripts. This should reduce the
@@ -259,23 +293,44 @@ proc ::htmlparse::parse {args} {
 
     set html [PrepareHtml [lindex $args 0]]
 
-    # Handle incomplete HTML
+    # Look for incomplete HTML from the last iteration and prepend it
+    # to the input we just got.
+
+    if {$incvar != {}} {
+	upvar $incvar incomplete
+    } else {
+	set incomplete ""
+    }
+
+    if {[catch {set new $incomplete$html}]} {set new $html}
+    set html $new
+
+    # Handle incomplete HTML (Recognize incomplete tag at end, buffer
+    # it up for the next call).
+
+    set end [lindex \{$html\} end]
+    if {[set idx [string last < $end]] > [string last > $end]} {
 
-    if {[regexp -- {[^<]*(<[^>]*)$} [lindex "\{$html\}" end] -> trailer]} {
 	if {$incvar == {}} {
 	    return -code error "::htmlparse::parse : HTML is incomplete, option -incvar is missing"
 	}
-	upvar $incvar incomplete
-	set incomplete $trailer
-	set html       [string range $html 0 [expr {[string last "<" $html] - 1}]]
+
+	#  upvar $incvar incomplete -- Already done, s.a.
+	set incomplete [string range $end $idx end]
+	incr idx -1
+	set html       [string range $end 0 $idx]
+	
+    } else {
+	set incomplete ""
     }
 
     # Convert the HTML string into a script.
 
-    set w " \t\r\n"	;# white space
-    set exp <(/?)([CClass ^$w>]+)[CClass $w]*([CClass ^>]*)>
+    set sub "\}\n$cmd {\\1} {} {\\2} \{\}\n$cmd {\\1} {/} {} \{"
+    regsub -all -- {<([^\s>]+)\s*([^>]*)/>} $html $sub html
+
     set sub "\}\n$cmd {\\2} {\\1} {\\3} \{"
-    regsub -all -- $exp $html $sub html
+    regsub -all -- {<(/?)([^\s>]+)\s*([^>]*)>} $html $sub html
 
     # The value of queue now determines wether we process the HTML by
     # ourselves (queue is empty) or if we generate a list of  scripts
@@ -341,20 +396,27 @@ proc ::htmlparse::PrepareHtml {html} {
     #
     # Also normalize the line endings (\r -> \n).
 
+    # Tcllib SF Bug 861287 - Processing of comments.
+    # Recognize EOC by RE, instead of fixed string.
+
     set html [string map [list \r \n] $html]
 
-    regsub -- "^.*<!DOCTYPE\[^>\]*>"       $html {}     html
-    regsub -all -- "-->"                $html "\001" html
-    regsub -all -- "<!--\[^\001\]*\001" $html {}     html
+    regsub -- "^.*<!DOCTYPE\[^>\]*>"    $html {}     html
+    regsub -all -- "--(\[ \t\n\]*)>"      $html "\001\\1\002" html
+
+    # Recognize borken beginnings of a comment and convert them to PCDATA.
+    regsub -all -- "<--(\[^\001\]*)\001(\[^\002\]*)\002" $html {\&lt;--\1--\2\&gt;} html
+
+    # And now recognize true comments, remove them.
+    regsub -all -- "<!--\[^\001\]*\001(\[^\002\]*)\002"  $html {}                   html
 
     # Protect characters special to tcl (braces, slashes) by
     # converting them to their escape sequences.
 
-    regsub -all -- \{   $html {\&ob;}  html
-    regsub -all -- \}   $html {\&cb;}  html
-    regsub -all -- \\\\ $html {\&bsl;} html
-
-    return $html
+    return [string map [list \
+		    "\{" "&#123;" \
+		    "\}" "&#125;" \
+		    "\\" "&#92;"] $html]
 }
 
 
@@ -404,7 +466,8 @@ proc ::htmlparse::debugCallback {args} {
 #
 #	Takes a HTML string, substitutes all escape sequences with
 #	their actual characters and returns the resulting string.
-#	HTML not containing escape sequences is returned unchanged.
+#	HTML not containing escape sequences or invalid escape
+#	sequences is returned unchanged.
 #
 # Arguments:
 #	html	The string to modify
@@ -417,66 +480,44 @@ proc ::htmlparse::debugCallback {args} {
 #	their actual characters.
 
 proc ::htmlparse::mapEscapes {html} {
-    # Find HTML escape characters of the form &xxx;
+    # Find HTML escape characters of the form &xxx(;|EOW)
 
-    if {![regexp -- & $html]} {
-	# HTML not containing escape sequences is returned unchanged.
-	return $html
-    }
-
-    regsub -all -- {([][$\\])} $html {\\\1} new
-    regsub -all -- {&#([0-9][0-9]?[0-9]?);?} $new {[format %c [scan \1 %d tmp;set tmp]]} new
-    regsub -all -- {&([a-zA-Z]+);?} $new {[DoMap \1]} new
+    # Quote special Tcl chars so they pass through [subst] unharmed.
+    set new [string map [list \] \\\] \[ \\\[ \$ \\\$ \\ \\\\] $html]
+    regsub -all -- {&([[:alnum:]]{2,7})(;|\M)} $new {[DoNamedMap \1 {\2}]} new
+    regsub -all -- {&#([[:digit:]]{1,5})(;|\M)} $new {[DoDecMap \1 {\2}]} new
+    regsub -all -- {&#x([[:xdigit:]]{1,4})(;|\M)} $new {[DoHexMap \1 {\2}]} new
     return [subst $new]
 }
 
-# htmlparse::CClass --
-#
-#	Internal helper command used by '::htmlparse::parse' while
-#	transforming the HTML string. Makes it easier to declare
-#	character classes in a ""-bounded string without traipsing
-#	into quoting hell.
-#
-# Arguments:
-#	x	A set of characters.
-#
-# Side Effects:
-#	None.
-#
-# Results:
-#	Returns a regular expression for the specified character
-#	class.
-
-proc ::htmlparse::CClass {x} {
-    return "\[$x\]"
+proc ::htmlparse::DoNamedMap {name endOf} {
+    variable namedEntities
+    if {[info exist namedEntities($name)]} {
+	return $namedEntities($name)
+    } else {
+	# Put it back..
+	return "&$name$endOf"
+    }
 }
 
-# htmlparse::DoMap --
-#
-#	Internal helper command. Takes a the body of a single escape
-#	sequence (i.e. the string without the sourounding & and ;) and
-#	returns the associated actual character. Used by
-#	'::htmlparse::mapEscapes' to do the real work.
-#
-# Arguments:
-#	text	The body of the escape sequence to convert.
-#
-#	unknown	Optional. Defaults to '?'. The string to return if the
-#		escape sequence is not known to the command.
-#
-# Side Effects:
-#	None.
-#
-# Results:
-#	None.
+proc ::htmlparse::DoDecMap {dec endOf} {
+    scan $dec %d dec
+    if {$dec <= 0xFFFD} {
+	return [format %c $dec]
+    } else {
+	# Put it back..
+	return "&#$dec$endOf"
+    }
+}
 
-proc ::htmlparse::DoMap {text {unknown ?}} {
-    # Convert an HTML escape sequence into a character.
-
-    variable escapes
-    set result $unknown
-    catch {set result $escapes($text)}
-    return $result
+proc ::htmlparse::DoHexMap {hex endOf} {
+    scan $hex %x value
+    if {$value <= 0xFFFD} {
+	return [format %c $value]
+    } else {
+	# Put it back..
+	return "&#x$hex$endOf"
+    }
 }
 
 # htmlparse::2tree --
@@ -516,14 +557,16 @@ proc ::htmlparse::2tree {html tree} {
 
     ::struct::stack ::htmlparse::tags
     ::htmlparse::tags push root
-    $tree set root -key type root
+    $tree set root type root
 
     parse -cmd [list ::htmlparse::2treeCallback $tree] $html
 
     # A bit hackish, correct the ordering of nodes for the optional
     # tag types, over a larger area when was seen by the parser itself.
 
-    $tree walk root -order post -command [list ::htmlparse::Reorder %t %n]
+    $tree walk root -order post n {
+	::htmlparse::Reorder $tree $n
+    }
 
     ::htmlparse::tags destroy
     return $tree
@@ -618,7 +661,7 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 		set level 1
 		set found 0
 		foreach n $nodes {
-		    set type [$tree get $n -key type]
+		    set type [$tree get $n type]
 		    if {0 == [string compare $tag $type]} {
 			# Found an earlier open tag -> (b).
 			set found 1
@@ -629,11 +672,11 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 		if {$found} {
 		    ::htmlparse::tags pop $level
 		    if {$level > 1} {
-			#foreach n $nodes {lappend tstring [$tree get $n -key type]}
+			#foreach n $nodes {lappend tstring [$tree get $n type]}
 			#puts stderr "\tdesync at <$tag> ($tstring) => pop $level"
 		    }
 		} else {
-		    #foreach n $nodes {lappend tstring [$tree get $n -key type]}
+		    #foreach n $nodes {lappend tstring [$tree get $n type]}
 		    #puts stderr "\tdesync at <$tag> ($tstring) => ignore"
 		}
 	    }
@@ -647,8 +690,8 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 	    # context.
 
 	    set        pcd  [$tree insert [::htmlparse::tags peek] end]
-	    $tree set $pcd  -key type PCDATA
-	    $tree set $pcd  -key data $textBehindTheTag
+	    $tree set $pcd  type PCDATA
+	    $tree set $pcd  data $textBehindTheTag
 	}
 
     } else {
@@ -665,8 +708,8 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 	# is ignored under all circcumstances.
 
 	set        node [$tree insert [::htmlparse::tags peek] end]
-	$tree set $node -key type $tag
-	$tree set $node -key data $param
+	$tree set $node type $tag
+	$tree set $node data $param
 
 	if {$textBehindTheTag != {}} {
 	    switch -exact -- $tag {
@@ -677,8 +720,8 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 		    set pcd  [$tree insert $node end]
 		}
 	    }
-	    $tree set $pcd  -key type PCDATA
-	    $tree set $pcd  -key data $textBehindTheTag
+	    $tree set $pcd type PCDATA
+	    $tree set $pcd data $textBehindTheTag
 	}
 
 	::htmlparse::tags push $node
@@ -716,9 +759,9 @@ proc ::htmlparse::2treeCallback {tree tag slash param textBehindTheTag} {
 #	None.
 
 proc ::htmlparse::removeVisualFluff {tree} {
-    $tree walk root \
-	    -order post \
-	    -command [list ::htmlparse::RemoveVisualFluff %t %n]
+    $tree walk root -order post n {
+	::htmlparse::RemoveVisualFluff $tree $n
+    }
     return
 }
 
@@ -739,9 +782,9 @@ proc ::htmlparse::removeVisualFluff {tree} {
 #	None.
 
 proc ::htmlparse::removeFormDefs {tree} {
-    $tree walk root \
-	    -order post \
-	    -command {::htmlparse::RemoveFormDefs %t %n}
+    $tree walk root -order post n {
+	::htmlparse::RemoveFormDefs $tree $n
+    }
     return
 }
 
@@ -761,7 +804,7 @@ proc ::htmlparse::removeFormDefs {tree} {
 #	None.
 
 proc ::htmlparse::RemoveVisualFluff {tree node} {
-    switch -exact -- [$tree get $node -key type] {
+    switch -exact -- [$tree get $node type] {
 	hmstart - html - font - center - div - sup - b - i {
 	    # Removes the node, but does not affect the nodes below
 	    # it. These are just made into chiildren of the parent of
@@ -795,7 +838,7 @@ proc ::htmlparse::RemoveVisualFluff {tree node} {
 #	None.
 
 proc ::htmlparse::RemoveFormDefs {tree node} {
-    switch -exact -- [$tree get $node -key type] {
+    switch -exact -- [$tree get $node type] {
 	form {
 	    $tree delete $node
 	}
@@ -823,7 +866,7 @@ proc ::htmlparse::RemoveFormDefs {tree node} {
 #	None.
 
 proc ::htmlparse::Reorder {tree node} {
-    switch -exact -- [set tp [$tree get $node -key type]] {
+    switch -exact -- [set tp [$tree get $node type]] {
 	h1 - h2 - h3 - h4 - h5 - h6 - p - li {
 	    # Look for right siblings until the next node with the
 	    # same type (or end of level) and move these below this
@@ -833,7 +876,7 @@ proc ::htmlparse::Reorder {tree node} {
 		set sibling [$tree next $node]
 		if {
 		    $sibling == {} ||
-		    (![string compare $tp [$tree get $sibling -key type]])
+		    (![string compare $tp [$tree get $sibling type]])
 		} {
 		    break
 		}
@@ -845,3 +888,7 @@ proc ::htmlparse::Reorder {tree node} {
 	}
     }
 }
+
+# ### ######### ###########################
+
+package provide htmlparse 1.1.1
