@@ -16,23 +16,15 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id$
+# $Id: md5x.tcl,v 1.14 2005/02/24 03:25:49 patthoyts Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 
-# Try and load a compiled extension to help.
-if {[catch {package require tcllibc}]} {
-    if {[catch {package require md5c}]} {
-        catch {
-            package requre Trf
-            package require Memchan
-        }
-    }
-}
-
 namespace eval ::md5 {
-    variable version 2.0.1
-    variable rcsid {$Id$}
+    variable version 2.0.4
+    variable rcsid {$Id: md5x.tcl,v 1.14 2005/02/24 03:25:49 patthoyts Exp $}
+    variable accel
+    array set accel {critcl 0 cryptkit 0 trf 0}
 
     namespace export md5 hmac MD5Init MD5Update MD5Final
 
@@ -50,18 +42,37 @@ namespace eval ::md5 {
 #   cleaned up when we call MD5Final
 #
 proc ::md5::MD5Init {} {
+    variable accel
     variable uid
     set token [namespace current]::[incr uid]
-    upvar #0 $token tok
+    upvar #0 $token state
 
     # RFC1321:3.3 - Initialize MD5 state structure
-    array set tok \
+    array set state \
         [list \
-             A [expr 0x67452301] \
-             B [expr 0xefcdab89] \
-             C [expr 0x98badcfe] \
-             D [expr 0x10325476] \
+             A [expr {0x67452301}] \
+             B [expr {0xefcdab89}] \
+             C [expr {0x98badcfe}] \
+             D [expr {0x10325476}] \
              n 0 i "" ]
+    if {$accel(cryptkit)} {
+        cryptkit::cryptCreateContext state(ckctx) CRYPT_UNUSED CRYPT_ALGO_MD5
+    } elseif {$accel(trf)} {
+        set s {}
+        switch -exact -- $::tcl_platform(platform) {
+            windows { set s [open NUL w] }
+            unix    { set s [open /dev/null w] }
+        }
+        if {$s != {}} {
+            fconfigure $s -translation binary -buffering none
+            ::md5 -attach $s -mode write \
+                -read-type variable \
+                -read-destination [subst $token](trfread) \
+                -write-type variable \
+                -write-destination [subst $token](trfwrite)
+            array set state [list trfread 0 trfwrite 0 trf $s]
+        }
+    }
     return $token
 }
 
@@ -76,30 +87,22 @@ proc ::md5::MD5Init {} {
 #   it here in preference to the pure-Tcl implementation.
 #
 proc ::md5::MD5Update {token data} {
-    variable $token
-    upvar 0 $token state
+    variable accel
+    upvar #0 $token state
 
-    if {[info command ::md5::md5c] != {}} {
+    if {$accel(critcl)} {
         if {[info exists state(md5c)]} {
             set state(md5c) [md5c $data $state(md5c)]
         } else {
             set state(md5c) [md5c $data]
         }
         return
-    } elseif {[package provide Trf] != {} \
-                  && [package provide Memchan] != {} \
-                  && 0 } {
-        # FIX ME - currently Trf usage is disabled by the above line.
-        # We have Trf and Memchan so we can create a bucket with these.
-        if {![info exists state(trf)]} {
-            set state(trf) [::null]
-            ::md5 -attach $state(trf) -mode write \
-                -read-type variable \
-                -read-destination [subst $token](trfread) \
-                -write-type variable \
-                -write-destination [subst $token](trfwrite)
-            fconfigure $state(trf) -translation binary -buffering none
+    } elseif {[info exists state(ckctx)]} {
+        if {[string length $data] > 0} {
+            cryptkit::cryptEncrypt $state(ckctx) $data
         }
+        return
+    } elseif {[info exists state(trf)]} {
         puts -nonewline $state(trf) $data
         return
     }
@@ -128,14 +131,23 @@ proc ::md5::MD5Update {token data} {
 #    Note that the output is 128 bits represented as binary data.
 #
 proc ::md5::MD5Final {token} {
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
 
     # Check for either of the C-compiled versions.
     if {[info exists state(md5c)]} {
         set r $state(md5c)
         unset state
         return $r
+    } elseif {[info exists state(ckctx)]} {
+        cryptkit::cryptEncrypt $state(ckctx) ""
+        cryptkit::cryptGetAttributeString $state(ckctx) \
+            CRYPT_CTXINFO_HASHVALUE r 16
+        cryptkit::cryptDestroyContext $state(ckctx)
+        # If nothing was hashed, we get no r variable set!
+        if {[info exists r]} {
+            unset state
+            return $r
+        }
     } elseif {[info exists state(trf)]} {
         close $state(trf)
         set r $state(trfwrite)
@@ -208,6 +220,7 @@ proc ::md5::HMACInit {K} {
     MD5Update $tok $Ki;                 # initialize with the inner pad
     
     # preserve the Ko value for the final stage.
+    # FRINK: nocheck
     set [subst $tok](Ko) $Ko
 
     return $tok
@@ -228,8 +241,7 @@ proc ::md5::HMACUpdate {token data} {
 #    closed and the binary representation of the hash result is returned.
 #
 proc ::md5::HMACFinal {token} {
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
 
     set tok [MD5Init];                  # init the outer hashing function
     MD5Update $tok $state(Ko);          # prepare with the outer pad.
@@ -490,17 +502,51 @@ if {[package provide Trf] != {}} {
     interp alias {} ::md5::Hex {} ::hex -mode encode --
 } else {
     proc ::md5::Hex {data} {
-        set result {}
-        binary scan $data c* r
-        foreach c $r {
-            append result [format "%02X" [expr {$c & 0xff}]]
-        }
-        return $result
+        binary scan $data H* result
+        return [string toupper $result]
     }
 }
 
 # -------------------------------------------------------------------------
 
+# LoadAccelerator --
+#
+#	This package can make use of a number of compiled extensions to
+#	accelerate the digest computation. This procedure manages the
+#	use of these extensions within the package. During normal usage
+#	this should not be called, but the test package manipulates the
+#	list of enabled accelerators.
+#
+proc ::md5::LoadAccelerator {name} {
+    variable accel
+    set r 0
+    switch -exact -- $name {
+        critcl {
+            if {![catch {package require tcllibc}]
+                || ![catch {package require md5c}]} {
+                set r [expr {[info command ::md5::md5c] != {}}]
+            }
+        }
+        cryptkit {
+            if {![catch {package require cryptkit}]} {
+                set r [expr {![catch {cryptkit::cryptInit}]}]
+            }
+        }
+        trf {
+            if {![catch {package require Trf}]} {
+                set r [expr {![catch {::md5 aa} msg]}]
+            }
+        }
+        default {
+            return -code error "invalid accelerator package:\
+                must be one of [join [array names accel] {, }]"
+        }
+    }
+    set accel($name) $r
+}
+
+# -------------------------------------------------------------------------
+
 # Description:
 #  Pop the nth element off a list. Used in options processing.
 #
@@ -516,8 +562,7 @@ proc ::md5::Pop {varname {nth 0}} {
 # fileevent handler for chunked file hashing.
 #
 proc ::md5::Chunk {token channel {chunksize 4096}} {
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     
     if {[eof $channel]} {
         fileevent $channel readable {}
@@ -537,11 +582,12 @@ proc ::md5::md5 {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0} { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
-                    must be one of $err"
+                    must be one of $err\nlen: [llength $args]"
             }
         }
         Pop args
@@ -565,6 +611,7 @@ proc ::md5::md5 {args} {
     } else {
 
         set tok [MD5Init]
+        # FRINK: nocheck
         set [subst $tok](reading) 1
         fileevent $opts(-channel) readable \
             [list [namespace origin Chunk] \
@@ -595,8 +642,9 @@ proc ::md5::hmac {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0} { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
                     must be one of $err"
@@ -628,6 +676,7 @@ proc ::md5::hmac {args} {
     } else {
 
         set tok [HMACInit $opts(-key)]
+        # FRINK: nocheck
         set [subst $tok](reading) 1
         fileevent $opts(-channel) readable \
             [list [namespace origin Chunk] \
@@ -649,6 +698,11 @@ proc ::md5::hmac {args} {
 
 # -------------------------------------------------------------------------
 
+# Try and load a compiled extension to help.
+namespace eval ::md5 {
+    foreach e {critcl cryptkit trf} { if {[LoadAccelerator $e]} { break } }
+}
+
 package provide md5 $::md5::version
 
 # -------------------------------------------------------------------------
