@@ -7,9 +7,9 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: pop3d.tcl,v 1.21 2005/09/28 04:51:23 andreas_kupries Exp $
 
-package require md5 1; # tcllib | APOP            | v1
+package require md5  ; # tcllib | APOP
 package require mime ; # tcllib | storage callback
 package require log  ; # tcllib | tracing
 
@@ -34,6 +34,7 @@ namespace eval ::pop3d {
     # sock    - listening socket
     # authCmd - authentication callback
     # storCmd - storage callback
+    # sockCmd - command prefix for openin the server socket
     # state   - state of the server (up, down, exiting)
     # conn    - map : sock -> state array
     # counter - counter for state arrays
@@ -62,10 +63,11 @@ namespace eval ::pop3d {
 	    "up"		\
 	    ]
 
-    variable version ; set version 1.0.2
+    variable version ; set version 1.1.0
     variable server  "tcllib/pop3d-$version"
 
     variable cmdMap ; array set cmdMap {
+	CAPA H_capa
 	USER H_user
 	PASS H_pass
 	APOP H_apop
@@ -78,6 +80,18 @@ namespace eval ::pop3d {
 	RSET H_rset
 	LIST H_list
     }
+
+    # Capabilities to be reported by the CAPA command. The list
+    # contains pairs of capability strings and the connection state in
+    # which they are reported. The state can be "auth", "trans", or
+    # "both".
+    variable capabilities \
+	[list \
+	     USER			both \
+	     PIPELINING			both \
+	     "IMPLEMENTATION $server"	trans \
+	    ]
+    
     # -- UIDL -- not implemented --
 
     # Only export one command, the one used to instantiate a new server
@@ -112,6 +126,7 @@ proc ::pop3d::new {{name ""}} {
 	variable port     110
 	variable trueport 110
 	variable sock     {}
+	variable sockCmd  socket
 	variable authCmd  {}
 	variable storCmd  {}
 	variable state    down
@@ -169,6 +184,7 @@ proc ::pop3d::_up {name} {
     upvar ::pop3d::pop3d::${name}::port     port
     upvar ::pop3d::pop3d::${name}::trueport trueport
     upvar ::pop3d::pop3d::${name}::state    state
+    upvar ::pop3d::pop3d::${name}::sockCmd  sockCmd
     upvar ::pop3d::pop3d::${name}::sock     sock
 
     log::log debug "pop3d $name up"
@@ -176,7 +192,10 @@ proc ::pop3d::_up {name} {
 
     log::log debug "pop3d $name listening, requested port $port"
 
-    set s [socket -server [list ::pop3d::HandleNewConnection $name] $port]
+    set cmd $sockCmd
+    lappend cmd -server [list ::pop3d::HandleNewConnection $name] $port
+    puts $cmd
+    set s [eval $cmd]
     set trueport [lindex [fconfigure $s -sockname] 2]
 
     ::log::log debug "pop3d $name listening on $trueport, socket $s ([fconfigure $s -sockname])"
@@ -285,10 +304,14 @@ proc ::pop3d::_cget {name anoption} {
 	    upvar ::pop3d::pop3d::${name}::storCmd storCmd
 	    return $storCmd
 	}
+	-socket {
+	    upvar ::pop3d::pop3d::${name}::sockCmd sockCmd
+	    return $sockCmd
+	}
 	default {
 	    return -code error \
 		    "Unknown option \"$anoption\":\
-		    Expected \"-state\", \"-port\", \"-auth\", or \"-storage\""
+		    Expected \"-state\", \"-port\", \"-auth\", \"-socket\", or \"-storage\""
 	}
     }
     # return - in all branches
@@ -319,6 +342,7 @@ proc ::pop3d::_configure {name args} {
     upvar ::pop3d::pop3d::${name}::port     port
     upvar ::pop3d::pop3d::${name}::authCmd  authCmd
     upvar ::pop3d::pop3d::${name}::storCmd  storCmd
+    upvar ::pop3d::pop3d::${name}::sockCmd  sockCmd
     upvar ::pop3d::pop3d::${name}::state    state
 
     if {$argc == 0} {
@@ -327,7 +351,8 @@ proc ::pop3d::_configure {name args} {
 		-port    $trueport \
 		-auth    $authCmd  \
 		-storage $storCmd  \
-		-state   $state
+		-socket  $sockCmd \
+		-state   $state \
 		]
     }
 
@@ -337,6 +362,7 @@ proc ::pop3d::_configure {name args} {
 	switch -exact -- $option {
 	    -auth    {set authCmd $value}
 	    -storage {set storCmd $value}
+	    -socket  {set sockCmd $value}
 	    -port    {
 		set port $value
 
@@ -353,7 +379,7 @@ proc ::pop3d::_configure {name args} {
 	    default {
 		return -code error \
 			"Unknown option \"$option\":\
-			Expected \"-port\", \"-auth\", or \"-storage\""
+			Expected \"-port\", \"-auth\", \"-socket\", or \"-storage\""
 	    }
 	}
 	set args [lrange $args 2 end]
@@ -421,13 +447,13 @@ proc ::pop3d::HandleNewConnection {name sock rHost rPort} {
     set cstate(msg)        0
     set cstate(size)       0
 
-    ::log::log notice "$name $sock state auth, waiting for logon"
+    ::log::log notice "pop3d $name $sock state auth, waiting for logon"
 
     fconfigure $sock -buffering line -translation crlf -blocking 0
 
     if {[catch {::pop3d::GreetPeer $name $sock} errmsg]} {
 	close $sock
-	log::log error "$name $sock greeting $errmsg"
+	log::log error "pop3d $name $sock greeting $errmsg"
 	unset cstate
 	unset conn($sock)
 	return
@@ -447,17 +473,17 @@ proc ::pop3d::CloseConnection {name sock} {
     # Kill a pending idle event for CloseConnection, we are closing now.
     catch {after cancel $cstate(idlepending)}
 
-    ::log::log debug "$name $sock closing connection"
+    ::log::log debug "pop3d $name $sock closing connection"
 
     if {[catch {close $sock} msg]} {
-	::log::log error "$name $sock close: $msg"
+	::log::log error "pop3d $name $sock close: $msg"
     }
     if {$storCmd != {}} {
 	# remove possible lock set in storage facility.
 	if {[catch {
 	    uplevel #0 [linsert $storCmd end unlock $cstate(storage)]
 	} msg]} {
-	    ::log::log error "$name $sock storage unlock: $msg"
+	    ::log::log error "pop3d $name $sock storage unlock: $msg"
 	    # -W- future ? kill all connections, execute clean up of storage
 	    # -W-          facility.
 	}
@@ -466,7 +492,7 @@ proc ::pop3d::CloseConnection {name sock} {
     unset cstate
     unset conn($sock)
 
-    ::log::log notice "$name $sock closed"
+    ::log::log notice "pop3d $name $sock closed"
 
     if {[string equal $state existing] && ([array size conn] == 0)} {
 	_destroy $name
@@ -495,7 +521,7 @@ proc ::pop3d::HandleCommand {name sock} {
     upvar $conn($sock)                   cstate
     variable                             cmdMap
 
-    ::log::log info "$name $sock < $line"
+    ::log::log info "pop3d $name $sock < $line"
 
     set fail [catch {
 	set cmd [string toupper [lindex $line 0]]
@@ -514,7 +540,7 @@ proc ::pop3d::HandleCommand {name sock} {
 	# Handled by closing the connection.
 	# (We do not know how to relay the internal error to the client)
 
-	::log::log error "$name $sock $cmd: $errmsg"
+	::log::log error "pop3d $name $sock $cmd: $errmsg"
 	CloseConnection $name $sock
     }
     return
@@ -543,8 +569,8 @@ proc ::pop3d::HandleUnknownCmd {name sock cmd line} {
 }
 
 proc ::pop3d::Respond2Client {name sock ok wtext} {
-    ::log::log info "$name $sock > $ok $wtext"
-    puts $sock                    "$ok $wtext"
+    ::log::log info "pop3d $name $sock > $ok $wtext"
+    puts $sock                          "$ok $wtext"
     return
 }
 
@@ -552,6 +578,29 @@ proc ::pop3d::Respond2Client {name sock ok wtext} {
 ##########################
 # Command implementations.
 
+proc ::pop3d::H_capa {name sock cmd line} {
+    # @c Handle CAPA command.
+
+    # Capabilities should better be configurable and handled per
+    # server object, so that e.g. USER/PASS authentication can be
+    # turned off.
+
+    upvar cstate cstate
+    variable capabilities
+
+    Respond2Client $name $sock +OK "Capability list follows"
+    foreach {capability state} $capabilities {
+	if {
+	    [string equal $state "both"] ||
+	    [string equal $state $cstate(state)]
+	} {
+	    puts $sock $capability
+	}
+    }
+    puts $sock .
+    return
+}
+
 proc ::pop3d::H_user {name sock cmd line} {
     # @c Handle USER command.
     #
@@ -606,14 +655,14 @@ proc ::pop3d::H_pass {name sock cmd line} {
 	set pwd [lindex [split $line] 1]
 
 	if {![uplevel #0 [linsert $authCmd end exists $cstate(name)]]} {
-	    ::log::log warning "$name $sock $authCmd lookup $cstate(name) : user does not exist"
+	    ::log::log warning "pop3d $name $sock $authCmd lookup $cstate(name) : user does not exist"
 	    CheckLogin $name $sock "" "" ""
 	    return
 	}
 	if {[catch {
 	    set info [uplevel #0 [linsert $authCmd end lookup $cstate(name)]]
 	} msg]} {
-	    ::log::log error "$name $sock $authCmd lookup $cstate(name) : $msg"
+	    ::log::log error "pop3d $name $sock $authCmd lookup $cstate(name) : $msg"
 	    CheckLogin $name $sock "" "" ""
 	    return
 	}
@@ -660,14 +709,14 @@ proc ::pop3d::H_apop {name sock cmd line} {
     set digest  [lindex $line 2]
 
     if {![uplevel #0 [linsert $authCmd end exists $cstate(name)]]} {
-	::log::log warning "$name $sock $authCmd lookup $cstate(name) : user does not exist"
+	::log::log warning "pop3d $name $sock $authCmd lookup $cstate(name) : user does not exist"
 	CheckLogin $name $sock "" "" ""
 	return
     }
     if {[catch {
 	set info [uplevel #0 [linsert $authCmd end lookup $cstate(name)]]
     } msg]} {
-	::log::log error "$name $sock $authCmd lookup $cstate(name) : $msg"
+	::log::log error "pop3d $name $sock $authCmd lookup $cstate(name) : $msg"
 	CheckLogin $name $sock "" "" ""
 	return
     }
@@ -675,7 +724,7 @@ proc ::pop3d::H_apop {name sock cmd line} {
     set pwd     [lindex $info 0]
     set storage [lindex $info 1]
 
-    ::log::log debug "$name $sock info = <$info>"
+    ::log::log debug "pop3d $name $sock info = <$info>"
 
     if {$storage == {}} {
 	# user does not exist, skip over digest computation
@@ -688,11 +737,11 @@ proc ::pop3d::H_apop {name sock cmd line} {
     # using tcl 8.x there is need to use channels, an immediate
     # computation is possible.
 
-    set ourDigest [md5::md5 "$cstate(id)$pwd"]
+    set ourDigest [Md5 "$cstate(id)$pwd"]
 
-    ::log::log debug "$name $sock digest input <$cstate(id)$pwd>"
-    ::log::log debug "$name $sock digest outpt <$ourDigest>"
-    ::log::log debug "$name $sock digest given <$digest>"
+    ::log::log debug "pop3d $name $sock digest input <$cstate(id)$pwd>"
+    ::log::log debug "pop3d $name $sock digest outpt <$ourDigest>"
+    ::log::log debug "pop3d $name $sock digest given <$digest>"
 
     CheckLogin $name $sock $digest $ourDigest $storage
     return
@@ -975,7 +1024,7 @@ proc ::pop3d::CheckLogin {name sock clientid serverid storage} {
 	set cstate(state) auth
 	set cstate(logon) ""
 
-	::log::log notice "$name $sock state auth, no maildrop"
+	::log::log notice "pop3d $name $sock state auth, no maildrop"
 	Respond2Client $name $sock -ERR "authentication failed, sorry"
 
     } elseif {[string compare $clientid $serverid] != 0} {
@@ -984,7 +1033,7 @@ proc ::pop3d::CheckLogin {name sock clientid serverid storage} {
 	set cstate(state) auth
 	set cstate(logon) ""
 
-	::log::log notice "$name $sock state auth, secret does not match"
+	::log::log notice "pop3d $name $sock state auth, secret does not match"
 	Respond2Client $name $sock -ERR "authentication failed, sorry"
 
     } elseif {
@@ -996,7 +1045,7 @@ proc ::pop3d::CheckLogin {name sock clientid serverid storage} {
 	set cstate(state) auth
 	set cstate(logon) ""
 
-	::log::log notice "$name $sock state auth, maildrop already locked"
+	::log::log notice "pop3d $name $sock state auth, maildrop already locked"
 	Respond2Client $name $sock -ERR \
 		"could not aquire lock for maildrop $cstate(name)"
     } else {
@@ -1015,8 +1064,8 @@ proc ::pop3d::CheckLogin {name sock clientid serverid storage} {
 	}
 	
 	::log::log notice \
-		"$name $sock login $cstate(name) $storage $cstate(msg)"
-	::log::log notice "$name $sock state trans"
+		"pop3d $name $sock login $cstate(name) $storage $cstate(msg)"
+	::log::log notice "pop3d $name $sock state trans"
 
 	Respond2Client $name $sock +OK "congratulations"
     }
@@ -1030,8 +1079,6 @@ proc ::pop3d::Transfer {name sock msgid {limit -1}} {
     upvar cstate cstate
     upvar ::pop3d::pop3d::${name}::storCmd storCmd
 
-    fileevent $sock readable {}
-
     if {$limit < 0} {
 	Respond2Client $name $sock +OK \
 		"[uplevel #0 [linsert $storCmd end \
@@ -1042,23 +1089,23 @@ proc ::pop3d::Transfer {name sock msgid {limit -1}} {
 
     set token [uplevel #0 [linsert $storCmd end get $cstate(storage) $msgid]]
     
-    ::log::log debug "$name $sock transfering data ($token)"
+    ::log::log debug "pop3d $name $sock transfering data ($token)"
 
     if {$limit < 0} {
 	# Full transfer, we can use "copymessage" and avoid
 	# construction in memory (depending on source of token).
 
-	log::log debug "$name Transfer $msgid /full"
-
-	#::mime::copymessage $token $sock
+	log::log debug "pop3d $name Transfer $msgid /full"
 
 	# We do "."-stuffing here. This is not in the scope of the
 	# MIME library we use, but a transport dependent thing.
 
-log::log debug "([string trimright [string map [list "\n." "\n.."] [mime::buildmessage $token]] \n])"
-
-	puts $sock [string trimright [string map [list "\n." "\n.."] [mime::buildmessage $token]] \n]
+	set msg [string trimright [string map [list "\n." "\n.."] \
+				       [mime::buildmessage $token]] \n]
+	log::log debug "($msg)"
+	puts $sock $msg
 	puts $sock .
+
     } else {
 	# As long as FR #531541 is not implemented we have to build
 	# the entire message in memory and then cut it down to the
@@ -1081,12 +1128,19 @@ log::log debug "([string trimright [string map [list "\n." "\n.."] [mime::buildm
 	regsub -- "\n\\.\n$" [string map [list "\n." "\n.."] [join [lrange $msg 0 $limit] \n]] {} data
 	puts $sock ${data}\n.
     }
-    fileevent $sock readable [list ::pop3d::HandleCommand $name $sock]
-    ::log::log debug "$name $sock transfer complete, listening again"
+    ::log::log debug "pop3d $name $sock transfer complete"
     # response already sent.
     return
 }
 
+set major [lindex [split [package require md5] .] 0]
+if {$::major < 2} {
+    proc ::pop3d::Md5 {text} {md5::md5 $text}
+} else {
+    proc ::pop3d::Md5 {text} {string tolower [md5::md5 -hex $text]}
+}
+unset major
+
 ##########################
 # Module initialization
 
