@@ -7,7 +7,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: graph.tcl,v 1.26 2005/09/30 05:36:39 andreas_kupries Exp $
 
 # Create the namespace before determining cgraph vs. tcl
 # Otherwise the loading 'struct.tcl' may get into trouble
@@ -22,19 +22,27 @@ namespace eval ::struct::graph {}
 # ** NOTE ** ATTENTION **
 #
 # For the 2.0 version of the graph interface 'cgraph 0.6' is _not_
-# useable anymore.
+# usable anymore.
 #
 # '[package vcompare $version 0.6] > 0' <=> '$version > 0.6'
 
+namespace eval ::struct::graph {}
+
 if {
-    ![catch {package require cgraph} version] &&
-    [package vcompare $version 0.6] > 0
+    ![catch {package require cgraph} ::struct::graph::version] &&
+    ([package vcompare $::struct::graph::version 0.6] > 0)
 } {
+    unset ::struct::graph::version
+
     # the cgraph package takes over, so we can return
+    namespace eval ::struct {
+	namespace import -force graph::*
+    }
     return
 }
 
-namespace eval ::struct {}
+package require struct::list
+
 namespace eval ::struct::graph {
     # Data storage in the graph module
     # -------------------------------
@@ -57,7 +65,6 @@ namespace eval ::struct::graph {
     # counter is used to give a unique name for unnamed graph
     variable counter 0
 
-
     # Only export one command, the one used to instantiate a new graph
     namespace export graph
 }
@@ -115,7 +122,7 @@ proc ::struct::graph::graph {args} {
     # FIRST, qualify the name.
     if {![string match "::*" $name]} {
         # Get caller's namespace; append :: if not global namespace.
-        set ns [uplevel 1 namespace current]
+        set ns [uplevel 1 [list namespace current]]
         if {"::" != $ns} {
             append ns "::"
         }
@@ -368,7 +375,7 @@ proc ::struct::graph::__arc_delete {name args} {
 
 # ::struct::graph::__arc_exists --
 #
-#	Test for existance of a given arc in a graph.
+#	Test for existence of a given arc in a graph.
 #
 # Arguments:
 #	name	name of the graph.
@@ -467,7 +474,7 @@ proc ::struct::graph::__arc_keys {name arc {pattern *}} {
 
 # ::struct::graph::__arc_keyexists --
 #
-#	Test for existance of a given key for a given arc in a graph.
+#	Test for existence of a given key for a given arc in a graph.
 #
 # Arguments:
 #	name	name of the graph.
@@ -846,7 +853,7 @@ proc ::struct::graph::__arc_unset {name arc key} {
 
     if {[array size data] == 0} {
 	# No attributes stored for this arc, squash the whole array.
-	set arcAttr($arc) {}
+	unset arcAttr($arc)
 	unset data
     }
     return
@@ -882,20 +889,38 @@ proc ::struct::graph::_arcs {name args} {
 	    -adj -
 	    -inner -
 	    -embedding {
+		if {$haveCond} {
+		    return -code error "invalid restriction:\
+			    illegal multiple use of\
+			    \"-in\"|\"-out\"|\"-adj\"|\"-inner\"|\"-embedding\""
+		}
+
 		set haveCond 1
 		set cond [string range $arg 1 end]
 	    }
 	    -key {
+		if {$haveKey} {
+		    return -code error {invalid restriction: illegal multiple use of "-key"}
+		}
+
 		incr i
 		set key [lindex $args $i]
 		set haveKey 1
 	    }
 	    -value {
+		if {$haveValue} {
+		    return -code error {invalid restriction: illegal multiple use of "-value"}
+		}
+
 		incr i
 		set value [lindex $args $i]
 		set haveValue 1
 	    }
 	    -filter {
+		if {$haveFilter} {
+		    return -code error {invalid restriction: illegal multiple use of "-filter"}
+		}
+
 		incr i
 		set fcmd [lindex $args $i]
 		set haveFilter 1
@@ -1277,7 +1302,7 @@ proc ::struct::graph::_keys {name {pattern *}} {
 
 # ::struct::graph::_keyexists --
 #
-#	Test for existance of a given key in a graph.
+#	Test for existence of a given key in a graph.
 #
 # Arguments:
 #	name	name of the graph.
@@ -1456,7 +1481,7 @@ proc ::struct::graph::__node_delete {name args} {
 
 # ::struct::graph::__node_exists --
 #
-#	Test for existance of a given node in a graph.
+#	Test for existence of a given node in a graph.
 #
 # Arguments:
 #	name	name of the graph.
@@ -1555,7 +1580,7 @@ proc ::struct::graph::__node_keys {name node {pattern *}} {
 
 # ::struct::graph::__node_keyexists --
 #
-#	Test for existance of a given key for a node in a graph.
+#	Test for existence of a given key for a node in a graph.
 #
 # Arguments:
 #	name	name of the graph.
@@ -1590,7 +1615,7 @@ proc ::struct::graph::__node_keyexists {name node key} {
 #			the routine will generate a unique node name.
 #
 # Results:
-#	node		The namee of the new node.
+#	node		The name of the new node.
 
 proc ::struct::graph::__node_insert {name args} {
 
@@ -1860,7 +1885,7 @@ proc ::struct::graph::__node_unset {name node key} {
 
     if {[array size data] == 0} {
 	# No attributes stored for this node, squash the whole array.
-	set nodeAttr($node) {}
+	unset nodeAttr($node)
 	unset data
     }
     return
@@ -1896,20 +1921,38 @@ proc ::struct::graph::_nodes {name args} {
 	    -adj -
 	    -inner -
 	    -embedding {
+		if {$haveCond} {
+		    return -code error "invalid restriction:\
+			    illegal multiple use of\
+			    \"-in\"|\"-out\"|\"-adj\"|\"-inner\"|\"-embedding\""
+		}
+
 		set haveCond 1
 		set cond [string range $arg 1 end]
 	    }
 	    -key {
+		if {$haveKey} {
+		    return -code error {invalid restriction: illegal multiple use of "-key"}
+		}
+
 		incr i
 		set key [lindex $args $i]
 		set haveKey 1
 	    }
 	    -value {
+		if {$haveValue} {
+		    return -code error {invalid restriction: illegal multiple use of "-value"}
+		}
+
 		incr i
 		set value [lindex $args $i]
 		set haveValue 1
 	    }
 	    -filter {
+		if {$haveFilter} {
+		    return -code error {invalid restriction: illegal multiple use of "-filter"}
+		}
+
 		incr i
 		set fcmd [lindex $args $i]
 		set haveFilter 1
@@ -2664,7 +2707,7 @@ proc ::struct::graph::Union {args} {
 
 # ::struct::graph::GenAttributeStorage --
 #
-#	Create an array to store the attrributes of a node in.
+#	Create an array to store the attributes of a node in.
 #
 # Arguments:
 #	name	Name of the graph containing the node
@@ -2694,6 +2737,12 @@ proc ::struct::graph::CheckSerialization {ser gavar navar aavar inavar outavar a
 	    $outavar outArcs   \
 	    $arcnvar arcNodes
 
+    array set nodeAttr  {}
+    array set arcAttr   {}
+    array set inArcs    {}
+    array set outArcs   {}
+    array set arcNodes  {}
+
     # Overall length ok ?
     if {[llength $ser] % 3 != 1} {
 	return -code error \
@@ -2726,7 +2775,7 @@ proc ::struct::graph::CheckSerialization {ser gavar navar aavar inavar outavar a
 	}
 
 	foreach arcd $narcs {
-	    foreach {arc dst attr} $arcd break
+	    foreach {arc dst aattr} $arcd break
 
 	    if {[info exists arcNodes($arc)]} {
 		return -code error \
@@ -2734,13 +2783,13 @@ proc ::struct::graph::CheckSerialization {ser gavar navar aavar inavar outavar a
 	    }
 
 	    # Attribute length ok ? Dictionary!
-	    if {[llength $attr] % 2} {
+	    if {[llength $aattr] % 2} {
 		return -code error \
 			"error in serialization: malformed arc attribute dictionary."
 	    }
 	    # Remember attribute data only for non-empty nodes
-	    if {[llength $attr]} {
-		set arcAttr($arc) $attr
+	    if {[llength $aattr]} {
+		set arcAttr($arc) $aattr
 	    }
 
 	    # Destination reference ok ?
@@ -2808,3 +2857,13 @@ proc ::struct::graph::luniq {list} {
     }
     return $result
 }
+
+# ### ### ### ######### ######### #########
+## Ready
+
+namespace eval ::struct {
+    # Get 'graph::graph' into the general structure namespace.
+    namespace import -force graph::graph
+    namespace export graph
+}
+package provide struct::graph 2.0.1
